﻿-- ------
--  YARA by Ameya Selené, a.k.a. Amenity (Andorhal_US)
--  http://www.curseforge.com/profiles/ameyaselene/
--  The author grants any and all persons the right to use this work for any non-commercial private purpose without condition
--  Distribution rights reserved by author under copyright law of the Federation of Saint Kitts and Nevis
--  UNAUTHORISED DISTRIBUTION AND/OR DUPLICATION OF THIS WORK IS EXPRESSLY PROHIBITED IN ALL CIRCUMSTANCES
--  ©2010-2011 Ameya Selené
-- ------


-- -----
--	PREP
-- -----
	--Only load for Death Knights
		if(select(2, UnitClass('player')) ~= 'DEATHKNIGHT') then return end
	
	--Kill default rune display
		local function YARADefaultFrameKill()
			if RuneFrame:IsVisible() then
				RuneFrame:UnregisterAllEvents()
				RuneFrame:Hide()
			end
		end
		
	--Simple function to round numbers
		local function round(number, decimals)
			return (("%%.%df"):format(decimals)):format(number)
		end
		

-- -----
--	LOCALS AND LIBRARIES
-- -----
	--Libraries
		local deflib = LibStub("LibDefaults")
		local LDB = LibStub("LibDataBroker-1.1")
		
	--Locals
		local addonName, L = ...; --Private table
		local runes = {};
		local RuneColors =
			{
				{1.0, 0.0, 0.0}, --Blood
				{0.0, 1.0, 0.0}, --Unholy
				{0.0, 1.0, 1.0}, --Frost
				{0.8, 0.1, 1.0}  --Death
			}
	

-- -----
--	DEFAULT OPTIONS
-- -----
	local DefOptions =
		{ 
			scale=1,
			OOCalpha=1,
			CombatFader=0,
			ICalpha=1,
			RPring=1,
			CDpeak=0.6,
			wedgerad=0
		}
		
	for k,v in pairs(DefOptions) do
		deflib:SetDefault(addonName, "YARAOpt", k, v)
	end
	
	
-- -----
--	FRAME GEN FUNCTIONS
-- -----
	--Main Parent
		local function YARAFrameInit()
			local frame = CreateFrame("Frame", addonName .. "Frame", UIParent)
			local tex = frame:CreateTexture(nil,"BACKGROUND")
			tex:SetTexture("Interface\\AddOns\\YARA\\img\\circle")
			tex:SetRotation(0)
			tex:SetVertexColor(0,0,0,1)
			tex:SetAllPoints(frame)
			frame:SetWidth(68);
			frame:SetHeight(68);
			frame:SetScale(YARAOpt.scale)
			frame:SetPoint("CENTER");
			frame:SetFrameStrata("BACKGROUND")
			frame:SetFrameLevel(4)
			frame:SetMovable(true)
			frame:SetUserPlaced(true)
			frame:SetScript("OnMouseDown", function() frame:StartMoving() end);
			frame:SetScript("OnMouseUp", function() frame:StopMovingOrSizing(); end);
			frame:SetClampedToScreen(true)
			frame:EnableMouse(false)
			frame:SetAlpha(YARAOpt.OOCalpha)
		end
		
		--LibDefault call to run YARAFrameInit after all defaults/vars loaded
			deflib:SetScript(addonName, YARAFrameInit)
			
	--Function to generate rune frames
		local function RuneFrameGen()
			for i = 1, 6 do
				local f = CreateFrame("Frame", addonName .. "Rune" .. i, YARAFrame);
				local rune = f:CreateTexture(nil, 'ARTWORK')
				local runetype = GetRuneType(i)
				rune:SetTexture("Interface\\AddOns\\YARA\\img\\wedge")
				if i == 3 then
					rune:SetRotation((YARAOpt.wedgerad*1.0471976)+1.0471976*4)
				elseif i == 4 then
					rune:SetRotation((YARAOpt.wedgerad*1.0471976)+1.0471976*3)
				else
					rune:SetRotation((YARAOpt.wedgerad*1.0471976)+1.0471976*i)
				end
				rune:SetVertexColor(unpack(RuneColors[runetype]))
				rune:SetAllPoints(f)
				f:SetWidth(66);
				f:SetHeight(66);
				--f:SetScale(YARAOpt.scale)
				f:SetPoint("CENTER", YARAFrame, "CENTER", 0, 0)
				runes[i] = rune;
			end
		end

		
-- -----
--	MAIN STUFF-DOING FUNCTIONS
-- -----	
	--Update rune status
		local function UpdateRunes(...)
			local runeID = ...
			if runeID <= 6 then
				local start, duration, runeReady = GetRuneCooldown(runeID)
				if runeReady == true then
					runes[runeID]:SetAlpha(1)
					UIFrameFadeIn(runes[runeID], 0, 1, 1)
				end
				if runeReady == false and (runes[runeID]:GetAlpha()) >= 1 then
					UIFrameFadeIn(runes[runeID], (start+duration-GetTime())-0.1, 0, YARAOpt.CDpeak)
				end
			end
		end
	
	--Function for Death Runes
		local function RuneTypeUpdater()
			for i = 1, 6 do
				local runeID = i
				local runetype = GetRuneType(runeID)
				runes[runeID]:SetVertexColor(unpack(RuneColors[runetype]));
			end
		end
	
	--Rune fader functions
		local function RuneFadeIn() --Going into combat
			if YARAOpt.CombatFader == 1 then
				UIFrameFadeIn(YARAFrame, 0.25, YARAOpt.OOCalpha, YARAOpt.ICalpha)
			end
			RuneTypeUpdater()
		end
		local function RuneFadeOut() --Coming out of combat
			if YARAOpt.CombatFader == 1 then
				UIFrameFadeIn(YARAFrame, 0.25, YARAOpt.ICalpha, YARAOpt.OOCalpha)
			end
		end
		
		
-- -----
--	EVENT HANDLER
-- -----
	--Event handler table (Thank you, Anticlaus!)
		local eventhandlers = 
			{
				PLAYER_LOGIN=RuneFrameGen,
				RUNE_POWER_UPDATE=UpdateRunes,
				RUNE_TYPE_UPDATE=RuneTypeUpdater,
				PLAYER_REGEN_DISABLED=RuneFadeIn,
				PLAYER_REGEN_ENABLED=RuneFadeOut,
				PLAYER_ENTERING_WORLD=YARADefaultFrameKill
			}
	
		local EventFrame = CreateFrame("FRAME")
		EventFrame:RegisterEvent("RUNE_POWER_UPDATE")
		EventFrame:RegisterEvent("RUNE_TYPE_UPDATE")
		EventFrame:RegisterEvent("PLAYER_LOGIN")
		EventFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
		EventFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
		EventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")

		EventFrame:SetScript("OnEvent", function(self, event, ...)
			if eventhandlers[event] then
				eventhandlers[event](...)
			end
		end)
		

-- _________________________________________________________________________________________________________
-- ---------------------------------------------------------------------------------------------------------
--	RUNIC POWER RING CODE
-- ---------------------------------------------------------------------------------------------------------
-- `````````````````````````````````````````````````````````````````````````````````````````````````````````


	--[[-------------------------------------------------------------------------
	rRingMod Copyright (c) 2009-2010, zork
	All rights reserved.
	  
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are
	met:
	  
		* Redistributions of source code must retain the above copyright
		  notice, this list of conditions and the following disclaimer.
		* Redistributions in binary form must reproduce the above
		  copyright notice, this list of conditions and the following
		  disclaimer in the documentation and/or other materials provided
		  with the distribution.
		* Neither the name of rRingMod nor the names of its contributors may
		  be used to endorse or promote products derived from this
		  software without specific prior written permission.
	  
	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
	A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
	OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
	LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES LOSS OF USE,
	DATA, OR PROFITS OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
	OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	---------------------------------------------------------------------------]]
  
  
	-- -----
	--  Modifications to rRingMod for use in YARA by Ameya Selené a.k.a. Amenity (Andorhal_US)
	--	http://www.curseforge.com/profiles/ameyaselene/
	--  The author grants any and all persons the right to use this work for any non-commercial private purpose without condition
	--  Distribution rights reserved by author under copyright law of the Federation of Saint Kitts and Nevis
	--  UNAUTHORISED DISTRIBUTION AND/OR DUPLICATION OF THIS WORK IS EXPRESSLY PROHIBITED IN ALL CIRCUMSTANCES
	--  ©2010-2011 Ameya Selené
	-- -----


-----------------------
-- CONFIG
-----------------------
	local ring_table
	  local function RingInit()
		  ring_table = {
			[1] = { 
			  global = {
				active = 1,
				unit = "player",
				ringname = "Remix_YARA_RPRing",
				size = 64,
				anchorframe = YARAFrame,
				anchorpoint = "CENTER",
				anchorposx = 0,
				anchorposy = 0,
				scale = 1,
				alpha = 1,
				framelevel = 3,
				gfx_folder = "rin",
				segments_used = 4,
				start_segment = 1,
				fill_direction = 1,
				ringtype = "mana",
			  },
			  background = {
				color = {r = 255/255, g = 255/255, b = 255/255, a = 1},
				alpha = 0.7,
				framelevel = 1,
				blendmode = "blend",
				use_texture = 0,
				do_rotation = 1,
				rotation = {
				  update_timer = 1/30,
				  step_size = 0.2,
				  direction = 1,
				},
			  },
			  foreground = {
				color = {r = 255/255, g = 255/255, b = 255/255, a = 1},
				alpha = 1,
				blendmode = "blend",
				framelevel = 3,
				use_texture = 0,
			  },
			  segment = {
				color = {r = 10/255, g = 150/255, b = 250/255, a = 1},
				alpha = 1,
				blendmode = "blend",
				framelevel = 2,
				segmentsize = 32,
				outer_radius = 27.5,
				inner_radius = 22.5,
			  },
			},
		}
	end 
  
  --Run above init function after defaults/vars loaded
	deflib:SetScript(addonName, RingInit)
  
---------------------  
-- FUNCTIONS
---------------------

  local function am(text)
    DEFAULT_CHAT_FRAME:AddMessage(text)
  end
  
  local function cre_ring_holder(ring_config)
    --am(ring_config.global.anchorframe)
    local f = CreateFrame("Frame",ring_config.global.ringname,ring_config.global.anchorframe)
    f:SetWidth(ring_config.global.size)
    f:SetHeight(ring_config.global.size)
    f:SetPoint(ring_config.global.anchorpoint,ring_config.global.anchorposx,ring_config.global.anchorposy)
    f:SetFrameLevel(ring_config.global.framelevel)
    f:SetScale(ring_config.global.scale)
    f:SetAlpha(ring_config.global.alpha)
    return f
  end
  
  local r2 = math.sqrt(0.5^2+0.5^2);
  
 
  local function cre_ring_background(ring_config, ring_object)
    local f = CreateFrame("Frame",nil,ring_object)
    f:SetAllPoints(ring_object)
    f:SetFrameLevel(ring_config.background.framelevel)
    f:SetAlpha(ring_config.background.alpha)
    if (ring_config.background.use_texture == 1) then
      local t = f:CreateTexture(nil,"BACKGROUND")
      t:SetTexture("Interface\\AddOns\\YARA\\img\\ring_background")
      t:SetAllPoints(f)
      t:SetVertexColor(ring_config.background.color.r,ring_config.background.color.g,ring_config.background.color.b,ring_config.background.color.a)
      t:SetBlendMode(ring_config.background.blendmode)
      f.texture = t
      --rotation
      if (ring_config.background.do_rotation == 1) then
        f.rotation = CreateFrame("Frame",nil,f)
        f.rotation.totalElapsed = 0
        f.rotation.direction = ring_config.background.rotation.direction
        if ring_config.background.rotation.direction == 1 then
          f.rotation.degrees = 0
        else 
          f.rotation.degrees = 360
        end
        f.rotation.step_size = ring_config.background.rotation.step_size
        f.rotation.update_timer = ring_config.background.rotation.update_timer
        f.rotation.starttime = GetTime()
        f:SetScript("OnUpdate", function (self,elapsed)
          do_background_rotation(f,elapsed)
        end)
      end
    end
    return f
  end
  
  local function cre_ring_foreground(ring_config, ring_object)
    local f = CreateFrame("Frame",nil,ring_object)
    f:SetAllPoints(ring_object)
    f:SetFrameLevel(ring_config.foreground.framelevel)
    f:SetAlpha(ring_config.foreground.alpha)
    if (ring_config.foreground.use_texture == 1) then
      local t = f:CreateTexture(nil,"foreground")
      t:SetTexture("Interface\\AddOns\\YARA\\img\\ring_foreground")
      t:SetAllPoints(f)
      t:SetVertexColor(ring_config.foreground.color.r,ring_config.foreground.color.g,ring_config.foreground.color.b,ring_config.foreground.color.a)
      t:SetBlendMode(ring_config.foreground.blendmode)
      f.texture = t
    end
    return f
  end
  
  -- calculates the one specific ring segment 
  local function calc_ring_segment(self, value)
  
    local t0 = self.square1
    local t1 = self.square2
    local t2 = self.slicer
    local t3 = self.fullsegment

    local direction = self.direction
    local segmentsize = self.segmentsize
    local outer_radius = self.outer_radius
    local difference = self.difference
    local inner_radius = self.inner_radius
    local ring_factor = self.ring_factor
    local ring_width = self.ring_width
    
    
    --remember to invert the value when direction is counter-clockwise
    local statusbarvalue = floor(value)
    if direction == 0 then
      statusbarvalue = 100 - statusbarvalue
    end
    if statusbarvalue == 0 then
      statusbarvalue = 1
      --am("Oh Oh")
    end

    --am(statusbarvalue)

    --angle
    local angle = statusbarvalue * 90 / 100
    local Arad = math.rad(angle)

    local Nx = 0
    local Ny = 0
    local Mx = segmentsize
    local My = segmentsize
    
    local Ix,Iy,Ox,Oy
    local IxCoord, IyCoord, OxCoord, OyCoord, NxCoord, NyCoord, MxCoord, MyCoord
    local sq1_c1_x, sq1_c1_y, sq1_c2_x, sq1_c2_y, sq1_c3_x, sq1_c3_y, sq1_c4_x, sq1_c4_y
    local sq2_c1_x, sq2_c1_y, sq2_c2_x, sq2_c2_y, sq2_c3_x, sq2_c3_y, sq2_c4_x, sq2_c4_y
    
    if direction == 1 then
      
      Ix = inner_radius * math.sin(Arad)
      Iy = (outer_radius - (inner_radius * math.cos(Arad))) + difference
      Ox = outer_radius * math.sin(Arad)
      Oy = (outer_radius - (outer_radius * math.cos(Arad))) + difference
      IxCoord = Ix / segmentsize 
      IyCoord = Iy / segmentsize
      OxCoord = Ox / segmentsize
      OyCoord = Oy / segmentsize   
      NxCoord = Nx / segmentsize
      NyCoord = Ny / segmentsize
      MxCoord = Nx / segmentsize
      MyCoord = Ny / segmentsize
      
      sq1_c1_x = NxCoord
      sq1_c1_y = NyCoord
      sq1_c2_x = NxCoord
      sq1_c2_y = IyCoord
      sq1_c3_x = IxCoord
      sq1_c3_y = NyCoord
      sq1_c4_x = IxCoord
      sq1_c4_y = IyCoord
            
      sq2_c1_x = IxCoord
      sq2_c1_y = NyCoord
      sq2_c2_x = IxCoord
      sq2_c2_y = OyCoord
      sq2_c3_x = OxCoord
      sq2_c3_y = NyCoord
      sq2_c4_x = OxCoord
      sq2_c4_y = OyCoord
      
      if self.field == 1 then
        t0:SetPoint("TOPLEFT",Nx,-Ny)
        t0:SetWidth(Ix)
        t0:SetHeight(Iy)
        t1:SetPoint("TOPLEFT",Ix,-Ny)
        t1:SetWidth(Ox-Ix)
        t1:SetHeight(Oy)
        t2:SetPoint("TOPLEFT",Ix,-Oy)
        t2:SetWidth(Ox-Ix)
        t2:SetHeight(Iy-Oy)
      elseif self.field == 2 then
        t0:SetPoint("TOPRIGHT",Nx,Ny)
        t0:SetWidth(Iy)
        t0:SetHeight(Ix)
        t1:SetPoint("TOPRIGHT",Ny,-Ix)
        t1:SetWidth(Oy)
        t1:SetHeight(Ox-Ix)
        t2:SetPoint("TOPRIGHT",-Oy,-Ix)
        t2:SetWidth(Iy-Oy)
        t2:SetHeight(Ox-Ix)
        t2:SetTexCoord(0,1, 1,1, 0,0, 1,0)
      elseif self.field == 3 then
        t0:SetPoint("BOTTOMRIGHT",Nx,Ny)
        t0:SetWidth(Ix)
        t0:SetHeight(Iy)
        t1:SetPoint("BOTTOMRIGHT",-Ix,Ny)
        t1:SetWidth(Ox-Ix)
        t1:SetHeight(Oy)
        t2:SetPoint("BOTTOMRIGHT",-Ix,Oy)
        t2:SetWidth(Ox-Ix)
        t2:SetHeight(Iy-Oy)
        t2:SetTexCoord(1,1, 1,0, 0,1, 0,0)
      elseif self.field == 4 then
        t0:SetPoint("BOTTOMLEFT",Nx,Ny)
        t0:SetWidth(Iy)
        t0:SetHeight(Ix)
        t1:SetPoint("BOTTOMLEFT",Ny,Ix)
        t1:SetWidth(Oy)
        t1:SetHeight(Ox-Ix)
        t2:SetPoint("BOTTOMLEFT",Oy,Ix)
        t2:SetWidth(Iy-Oy)
        t2:SetHeight(Ox-Ix)
        t2:SetTexCoord(1,0, 0,0, 1,1, 0,1)
      end
      
    else
      
      Ix = inner_radius * math.sin(Arad)
      Iy = (outer_radius - (inner_radius * math.cos(Arad))) + difference
      Ox = outer_radius * math.sin(Arad)
      Oy = (outer_radius - (outer_radius * math.cos(Arad))) + difference
      IxCoord = Ix / segmentsize 
      IyCoord = Iy / segmentsize
      OxCoord = Ox / segmentsize
      OyCoord = Oy / segmentsize   
      NxCoord = Nx / segmentsize
      NyCoord = Ny / segmentsize
      MxCoord = Mx / segmentsize
      MyCoord = My / segmentsize
      
      sq1_c1_x = IxCoord
      sq1_c1_y = IyCoord
      sq1_c2_x = IxCoord
      sq1_c2_y = MyCoord
      sq1_c3_x = MxCoord
      sq1_c3_y = IyCoord
      sq1_c4_x = MxCoord
      sq1_c4_y = MyCoord
            
      sq2_c1_x = OxCoord
      sq2_c1_y = OyCoord
      sq2_c2_x = OxCoord
      sq2_c2_y = IyCoord
      sq2_c3_x = MxCoord
      sq2_c3_y = OyCoord
      sq2_c4_x = MxCoord
      sq2_c4_y = IyCoord
      
      if self.field == 1 then
        t0:SetPoint("TOPLEFT",Ix,-Iy)
        t0:SetWidth(segmentsize-Ix)
        t0:SetHeight(segmentsize-Iy)
        t1:SetPoint("TOPLEFT",Ox,-Oy)
        t1:SetWidth(segmentsize-Ox)
        t1:SetHeight(Iy-Oy)
        t2:SetPoint("TOPLEFT",Ix,-Oy)
        t2:SetWidth(Ox-Ix)
        t2:SetHeight(Iy-Oy)
      elseif self.field == 2 then
        t0:SetPoint("TOPRIGHT",-Iy,-Ix)
        t0:SetWidth(segmentsize-Iy)
        t0:SetHeight(segmentsize-Ix)
        t1:SetPoint("TOPRIGHT",-Oy,-Ox)
        t1:SetWidth(Iy-Oy)
        t1:SetHeight(segmentsize-Ox)
        t2:SetPoint("TOPRIGHT",-Oy,-Ix)
        t2:SetWidth(Iy-Oy)
        t2:SetHeight(Ox-Ix)
        t2:SetTexCoord(0,1, 1,1, 0,0, 1,0)
      elseif self.field == 3 then
        t0:SetPoint("BOTTOMRIGHT",-Ix,Iy)
        t0:SetWidth(segmentsize-Ix)
        t0:SetHeight(segmentsize-Iy)
        t1:SetPoint("BOTTOMRIGHT",-Ox,Oy)
        t1:SetWidth(segmentsize-Ox)
        t1:SetHeight(Iy-Oy)
        t2:SetPoint("BOTTOMRIGHT",-Ix,Oy)
        t2:SetWidth(Ox-Ix)
        t2:SetHeight(Iy-Oy)
        t2:SetTexCoord(1,1, 1,0, 0,1, 0,0)
      elseif self.field == 4 then
        t0:SetPoint("BOTTOMLEFT",Iy,Ix)
        t0:SetWidth(segmentsize-Iy)
        t0:SetHeight(segmentsize-Ix)
        t1:SetPoint("BOTTOMLEFT",Oy,Ox)
        t1:SetWidth(Iy-Oy)
        t1:SetHeight(segmentsize-Ox)
        t2:SetPoint("BOTTOMLEFT",Oy,Ix)
        t2:SetWidth(Iy-Oy)
        t2:SetHeight(Ox-Ix)
        t2:SetTexCoord(1,0, 0,0, 1,1, 0,1)
      end
    end
    
    if self.field == 1 then
      --1,2,3,4
      t0:SetTexCoord(sq1_c1_x,sq1_c1_y, sq1_c2_x,sq1_c2_y, sq1_c3_x,sq1_c3_y, sq1_c4_x, sq1_c4_y)
      t1:SetTexCoord(sq2_c1_x,sq2_c1_y, sq2_c2_x,sq2_c2_y, sq2_c3_x,sq2_c3_y, sq2_c4_x, sq2_c4_y)
    elseif self.field == 2 then
      --2,4,1,3
      t0:SetTexCoord(sq1_c2_x,sq1_c2_y, sq1_c4_x, sq1_c4_y, sq1_c1_x,sq1_c1_y, sq1_c3_x,sq1_c3_y)
      t1:SetTexCoord(sq2_c2_x,sq2_c2_y, sq2_c4_x, sq2_c4_y, sq2_c1_x,sq2_c1_y, sq2_c3_x,sq2_c3_y)
    elseif self.field == 3 then
      --4,3,2,1
      t0:SetTexCoord(sq1_c4_x, sq1_c4_y, sq1_c3_x,sq1_c3_y, sq1_c2_x,sq1_c2_y, sq1_c1_x,sq1_c1_y)
      t1:SetTexCoord(sq2_c4_x, sq2_c4_y, sq2_c3_x,sq2_c3_y, sq2_c2_x,sq2_c2_y, sq2_c1_x,sq2_c1_y)
    elseif self.field == 4 then
      --3,1,4,2
      t0:SetTexCoord(sq1_c3_x,sq1_c3_y, sq1_c1_x,sq1_c1_y, sq1_c4_x, sq1_c4_y, sq1_c2_x,sq1_c2_y)
      t1:SetTexCoord(sq2_c3_x,sq2_c3_y, sq2_c1_x,sq2_c1_y, sq2_c4_x, sq2_c4_y, sq2_c2_x,sq2_c2_y)
    end

  
  end
  
  --function that creates the textures for each segment
  local function cre_segment_textures(ring_config,self)
    --am(self.field)
    
    local direction = ring_config.global.fill_direction
    local segmentsize = ring_config.segment.segmentsize
    local outer_radius = ring_config.segment.outer_radius
    local difference = segmentsize-outer_radius
    local inner_radius = ring_config.segment.inner_radius
    local ring_factor = outer_radius/inner_radius
    local ring_width = outer_radius-inner_radius
    
    self.direction =  ring_config.global.fill_direction
    self.segmentsize = ring_config.segment.segmentsize
    self.outer_radius = ring_config.segment.outer_radius
    self.difference = segmentsize-outer_radius
    self.inner_radius = ring_config.segment.inner_radius
    self.ring_factor = outer_radius/inner_radius
    self.ring_width = outer_radius-inner_radius
    
    local t0 = self:CreateTexture(nil, "BACKGROUND")
    t0:SetTexture("Interface\\AddOns\\YARA\\img\\ring_segment")
    t0:SetVertexColor(ring_config.segment.color.r,ring_config.segment.color.g,ring_config.segment.color.b,ring_config.segment.color.a)
    t0:SetBlendMode(ring_config.segment.blendmode)
    t0:Hide()
    
    local t1 = self:CreateTexture(nil, "LOW")
    t1:SetTexture("Interface\\AddOns\\YARA\\img\\ring_segment")
    t1:SetVertexColor(ring_config.segment.color.r,ring_config.segment.color.g,ring_config.segment.color.b,ring_config.segment.color.a)
    t1:SetBlendMode(ring_config.segment.blendmode)
    t1:Hide()

    local t2 = self:CreateTexture(nil, "BACKGROUND")
    t2:SetVertexColor(ring_config.segment.color.r,ring_config.segment.color.g,ring_config.segment.color.b,ring_config.segment.color.a)
    t2:SetBlendMode(ring_config.segment.blendmode)
    if direction == 1 then
      t2:SetTexture("Interface\\AddOns\\YARA\\img\\slicer1")
    else
      t2:SetTexture("Interface\\AddOns\\YARA\\img\\slicer0")
    end
    t2:Hide()

    local t3 = self:CreateTexture(nil, "BACKGROUND")
    t3:SetTexture("Interface\\AddOns\\YARA\\img\\ring_segment")
    t3:SetVertexColor(ring_config.segment.color.r,ring_config.segment.color.g,ring_config.segment.color.b,ring_config.segment.color.a)
    t3:SetBlendMode(ring_config.segment.blendmode)
    t3:SetPoint("CENTER",0,0)
    t3:SetWidth(segmentsize)
    t3:SetHeight(segmentsize)
    if self.field == 1 then
      --no coord needed
    elseif self.field == 2 then
      t3:SetTexCoord(0,1, 1,1, 0,0, 1,0)
    elseif self.field == 3 then
      t3:SetTexCoord(1,1, 1,0, 0,1, 0,0)
    elseif self.field == 4 then
      t3:SetTexCoord(1,0, 0,0, 1,1, 0,1)
    end
    t3:Hide()
    
    self.square1 = t0
    self.square2 = t1
    self.slicer = t2
    self.fullsegment = t3
    
  end
  
  --calculate the segment number based on starting segment and direction
  local function calc_segment_num(ring_config,current)
    local start = ring_config.global.start_segment
    local dir = ring_config.global.fill_direction
    local id
    if dir == 0 then
      if start-current < 1 then
        id = start-current+4
      else
        id = start-current
      end
    else
      if start+current > 4 then
        id = start+current-4
      else
        id = start+current
      end          
    end
    return id
  end
  
  --create the ring segments
  local function cre_ring_segments(ring_config, ring_object)
    local f = CreateFrame("Frame",nil,ring_object)
    f:SetAllPoints(ring_object)
    f:SetFrameLevel(ring_config.segment.framelevel)
    f:SetAlpha(ring_config.segment.alpha)
    for i=1,(ring_config.global.segments_used) do
      f[i] = CreateFrame("Frame",nil,f)
      f[i]:SetWidth(ring_config.global.size/2)
      f[i]:SetHeight(ring_config.global.size/2)
      f[i].id = i
      f[i].field = calc_segment_num(ring_config,i-1)
      --am(f[i].id.." ~ "..f[i].field)
      if f[i].field == 1 then
        f[i]:SetPoint("TOPRIGHT",0,0)
      elseif f[i].field == 2 then
        f[i]:SetPoint("BOTTOMRIGHT",0,0)
      elseif f[i].field == 3 then
        f[i]:SetPoint("BOTTOMLEFT",0,0)
      elseif f[i].field == 4 then
        f[i]:SetPoint("TOPLEFT",0,0)
      end
      cre_segment_textures(ring_config,f[i])
    end
    return f
  end

  
  local function calc_ring_mana(self,ring_config,unit)
    local act, max, perc, perc_per_seg = UnitPower(unit), UnitPowerMax(unit), floor((UnitPower(unit)/UnitPowerMax(unit))*100), 100/ring_config.global.segments_used
    local anz_seg, sum_radius = ring_config.global.segments_used, ring_config.global.segments_used*90
    
--"Stretchy Texture" at 25, 50, and 75 fix
	if perc == 25 or perc == 50 or perc == 75 then
		perc = perc+1
	end
    if perc == 0 or UnitIsDeadOrGhost(unit) == 1 then
      for i=1, anz_seg do
        self.segments[i].square1:Hide()
        self.segments[i].square2:Hide()
        self.segments[i].slicer:Hide()
        self.segments[i].fullsegment:Hide()
      end
    elseif perc == 100 then
      for i=1, anz_seg do
        self.segments[i].square1:Hide()
        self.segments[i].square2:Hide()
        self.segments[i].slicer:Hide()
        self.segments[i].fullsegment:Show()
      end
    else
      for i=1, anz_seg do
        if(perc > (i*perc_per_seg)) then
          self.segments[i].square1:Hide()
          self.segments[i].square2:Hide()
          self.segments[i].slicer:Hide()
          self.segments[i].fullsegment:Show()
        elseif ((perc >= ((i-1)*perc_per_seg)) and (perc <= (i*perc_per_seg))) then
          local value = ((perc-((i-1)*perc_per_seg))/perc_per_seg)*100
          calc_ring_segment(self.segments[i],value)
          self.segments[i].square1:Show()
          self.segments[i].square2:Show()
          self.segments[i].slicer:Show()
          self.segments[i].fullsegment:Hide()
        else
          self.segments[i].square1:Hide()
          self.segments[i].square2:Hide()
          self.segments[i].slicer:Hide()
          self.segments[i].fullsegment:Hide()
        end
      end
    end
  end  
  
  local function setup_rings(id)
	if YARAOpt.RPring == 0 then return end
	
    local ring_config = ring_table[id]
    --am(ring_config.global.ringname)
    --am(ring_config.global.unit)
    
    if ring_config.global.active == 1 then
    
      local ring_object = cre_ring_holder(ring_config)
      ring_object.background = cre_ring_background(ring_config, ring_object)
      ring_object.segments = cre_ring_segments(ring_config, ring_object)
      ring_object.foreground = cre_ring_foreground(ring_config, ring_object)
      --am(ring_object.background.texture:GetTexture())
      --am(ring_object.segments[1].field)
      
      if ring_config.global.ringtype == "health" then
        ring_object:SetScript("OnEvent", function(self, event, unit)
          if (event == "UNIT_HEALTH" and unit == ring_config.global.unit) or event == "PLAYER_ENTERING_WORLD" then
            calc_ring_health(ring_object,ring_config,ring_config.global.unit)
          end    
        end)
        ring_object:RegisterEvent("UNIT_HEALTH")
        ring_object:RegisterEvent("PLAYER_ENTERING_WORLD")
      end
      

	  
      if ring_config.global.ringtype == "mana" then
        ring_object:SetScript("OnEvent", function(self, event, unit)
			if (event == "UNIT_POWER" and unit == ring_config.global.unit) or event == "PLAYER_ENTERING_WORLD" then
				calc_ring_mana(ring_object,ring_config,ring_config.global.unit)
			end
        end)
		
		
		ring_object:RegisterEvent("UNIT_POWER")
        ring_object:RegisterEvent("PLAYER_ENTERING_WORLD")
      end
    end

  end
  
  
  ---------------------  
  -- CALL
  ---------------------
  
  local a = CreateFrame("Frame", nil, UIParent)
  
  a:RegisterEvent("ADDON_LOADED")
  
  a:SetScript("OnEvent", function (self,event,arg1)
    if(event=="ADDON_LOADED") and arg1 == addonName then
      --am("RingMod loaded.")
      for i in ipairs(ring_table) do 
        setup_rings(i)
      end
    end
  end)  
  
  
	-- _________________________________________________________________________________________________________
	-- ---------------------------------------------------------------------------------------------------------
	--	END RING CODE
	-- ---------------------------------------------------------------------------------------------------------
	-- `````````````````````````````````````````````````````````````````````````````````````````````````````````


-- -----
--	ADDON OPTIONS
-- -----
	
	--Slash handler
		_G["SlashCmdList"]["REMIX_YARA_MAIN"] = function() InterfaceOptionsFrame_OpenToCategory("|cffFF0000Y|r|cff00ffffA|r|cff00ff00R|r|cffff00ffA|r") end
		_G["SLASH_REMIX_YARA_MAIN1"] = "/yara"
		
	--Main options panel
		YARA = {};
		 YARA.panel = CreateFrame( "Frame", "YARAconfig", InterfaceOptionsFramePanelContainer );
		 -- Register in the Interface Addon Options GUI
		 -- Set the name for the Category for the Options Panel
		 YARA.panel.name = "|cffFF0000Y|r|cff00ffffA|r|cff00ff00R|r|cffff00ffA|r";
		 -- Add the panel to the Interface Options
		 InterfaceOptions_AddCategory(YARA.panel);
		 
		local title = YARA.panel:CreateFontString("YARAConfigTitle", "ARTWORK", "GameFontNormalLarge")
		title:SetPoint("TOPLEFT", 16, -16)
		title:SetText("|cffFF0000Y|r|cff00ffffA|r|cff00ff00R|r|cffff00ffA|r")
		local desc = YARA.panel:CreateFontString("YARAConfigDesc", "ARTWORK", "GameFontNormalSmall")
		desc:SetPoint("TOPLEFT", 16, -32)
		desc:SetText(L["Yet Another Rune Addon!"])
		desc:SetTextColor(1,1,1,1)
	
	--Scale Slider
	
		local scalesliderText
		local scaleslider = CreateFrame("Slider", "YARAScaleSlider", YARA.panel, "OptionsSliderTemplate")
		scaleslider:SetPoint("TOPLEFT", 16, -80)
		scaleslider:SetWidth(125)
		scaleslider:SetMinMaxValues(0.1, 6)
		scaleslider:SetValueStep(0.05)
		scaleslider:SetScript("OnValueChanged", function(self, value)
			YARAFrame:SetScale(value)
			YARAOpt.scale = YARAFrame:GetScale()
			scalesliderText:SetText(round(scaleslider:GetValue(), 2))
		end)
		scaleslider:SetScript("OnShow", function(frame)
			scaleslider:SetValue(YARAOpt.scale)
		end)
		
		scaleslider:EnableMouseWheel(1)
		scaleslider:SetScript("OnMouseWheel", function(arg1, arg2) --Mousewheel function
			if arg2 == 1 then
				local value = YARAFrame:GetScale()
				YARAFrame:SetScale(value + 0.05)
				YARAOpt.scale = YARAFrame:GetScale()
				scaleslider:SetValue(YARAOpt.scale)
				scalesliderText:SetText(round(YARAFrame:GetScale(), 2))
			elseif arg2 == -1 then
				local value = YARAFrame:GetScale()
				YARAFrame:SetScale(value - 0.05)
				YARAOpt.scale = YARAFrame:GetScale()
				scaleslider:SetValue(YARAOpt.scale)
				scalesliderText:SetText(round(YARAFrame:GetScale(), 2))
			end
		end)
				
		scalesliderText = scaleslider:CreateFontString("YARAScaleSliderTitle", "ARTWORK", "GameFontNormal")
		scalesliderText:SetPoint("TOP", scaleslider, "BOTTOM", 0, 0)
		scalesliderText:SetText(round(scaleslider:GetValue(), 1))
		--scalesliderText:SetTextColor(1,1,1,1)
		getglobal(scaleslider:GetName() .. 'Low'):SetText('0.1')
		getglobal(scaleslider:GetName() .. 'High'):SetText('6.0')
		getglobal(scaleslider:GetName() .. 'Text'):SetText(L['Scale'])
		getglobal(scaleslider:GetName() .. 'Text'):SetTextColor(1,1,0,1)
		
	--Lock Button
		local btnLock = CreateFrame("CheckButton", "YARALockButton", YARA.panel)
		btnLock:SetWidth(26)
		btnLock:SetHeight(26)
		btnLock:SetPoint("TOPLEFT", scaleslider, "BOTTOMLEFT", 0, -40)
		
		btnLock:SetScript("OnShow", function(frame)
			if YARAFrame:IsMouseEnabled() then
				frame:SetChecked(false)
			else
				frame:SetChecked(true)
			end
		end)
		
		btnLock:SetScript("OnClick", function(frame)
			local tick = frame:GetChecked()
			if tick then
				PlaySound("igMainMenuOptionCheckBoxOn")
				YARAFrame:EnableMouse(false)
				print("|cffFF0000Y|r|cff00ffffA|r|cff00ff00R|r|cffff00ffA|r: |cff00ffff"..L["Frame is now locked into place!"].."|r")
			else
				PlaySound("igMainMenuOptionCheckBoxOff")
				YARAFrame:EnableMouse(true)
				InterfaceOptionsFrame:Hide()
				print("|cffFF0000Y|r|cff00ffffA|r|cff00ff00R|r|cffff00ffA|r: |cff00ffff"..L["Frame unlocked! Enter '/yara' to return to the options menu."].."|r")
			end
		end)

		btnLock:SetHitRectInsets(0, -100, 0, 0)

		btnLock:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Up")
		btnLock:SetPushedTexture("Interface\\Buttons\\UI-CheckBox-Down")
		btnLock:SetHighlightTexture("Interface\\Buttons\\UI-CheckBox-Highlight")
		btnLock:SetCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check")

		local btnLockText = btnLock:CreateFontString("YARALockButtonTitle", "ARTWORK", "GameFontNormal")
		btnLockText:SetPoint("LEFT", btnLock, "RIGHT", 0, 1)
		btnLockText:SetText((L["Lock YARA"]):format(COMPLAINT_ADDED))

	--Runic Power Ring Option
		local btnRPRing = CreateFrame("CheckButton", "YARARPRingButton", YARA.panel)
		btnRPRing:SetWidth(26)
		btnRPRing:SetHeight(26)
		btnRPRing:SetPoint("TOPLEFT", btnLock, "BOTTOMLEFT", 0, -20)
		
		btnRPRing:SetScript("OnShow", function(frame)
			if YARAOpt.RPring == 0 then
				frame:SetChecked(false)
			else
				frame:SetChecked(true)
			end
		end)
		btnRPRing:SetScript("OnClick", function(frame)
			local tick = frame:GetChecked()
			if tick then
				PlaySound("igMainMenuOptionCheckBoxOn")
				YARAOpt.RPring = 1
				print("|cffFF0000Y|r|cff00ffffA|r|cff00ff00R|r|cffff00ffA|r: |cff00ffff"..L["Runic Power ring enabled. Please reload your UI for this change to take effect!"].."|r")
			else
				PlaySound("igMainMenuOptionCheckBoxOff")
				YARAOpt.RPring = 0
				print("|cffFF0000Y|r|cff00ffffA|r|cff00ff00R|r|cffff00ffA|r: |cff00ffff"..L["Runic Power ring disabled. Please reload your UI for this change to take effect!"].."|r")
			end
		end)

		btnRPRing:SetHitRectInsets(0, -100, 0, 0)

		btnRPRing:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Up")
		btnRPRing:SetPushedTexture("Interface\\Buttons\\UI-CheckBox-Down")
		btnRPRing:SetHighlightTexture("Interface\\Buttons\\UI-CheckBox-Highlight")
		btnRPRing:SetCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check")

		local btnRPRingText = btnRPRing:CreateFontString("YARARPRingButtonTitle1", "ARTWORK", "GameFontNormal")
		btnRPRingText:SetPoint("LEFT", btnRPRing, "RIGHT", 0, 1)
		btnRPRingText:SetText((L["Runic Power Ring"]):format(COMPLAINT_ADDED))
		
		local btnRPRingText2 = btnRPRing:CreateFontString("YARARPRingButtonTitle2", "ARTWORK", "GameFontHighlightSmall")
		btnRPRingText2:SetPoint("TOPLEFT", btnRPRing, "BOTTOMLEFT", 0, -1)
		btnRPRingText2:SetText((L["(Requires reload to take effect!)"]))
	
	--OOC Alpha Slider
		local OOCAlphaSliderText
		local OOCAlphaSlider = CreateFrame("Slider", "OOCAlphaSlider", YARA.panel, "OptionsSliderTemplate")
		OOCAlphaSlider:SetPoint("TOPLEFT", scaleslider, "TOPRIGHT", 100, 0)
		OOCAlphaSlider:SetWidth(125)
		OOCAlphaSlider:SetMinMaxValues(0, 1)
		OOCAlphaSlider:SetValueStep(0.1)
		OOCAlphaSlider:SetScript("OnValueChanged", function(self, value)
			YARAFrame:SetAlpha(value)
			YARAOpt.OOCalpha = YARAFrame:GetAlpha()
			OOCAlphaSliderText:SetText(round(OOCAlphaSlider:GetValue(), 1))
		end)
		OOCAlphaSlider:SetScript("OnShow", function(frame)
			OOCAlphaSlider:SetValue(YARAOpt.OOCalpha)
		end)
		
		OOCAlphaSliderText = OOCAlphaSlider:CreateFontString("YARAConfigSlider1Title", "ARTWORK", "GameFontNormal")
		OOCAlphaSliderText:SetPoint("TOP", OOCAlphaSlider, "BOTTOM", 0, 0)
		OOCAlphaSliderText:SetText(round(OOCAlphaSlider:GetValue(), 1))
		
		getglobal(OOCAlphaSlider:GetName() .. 'Low'):SetText('0')
		getglobal(OOCAlphaSlider:GetName() .. 'High'):SetText('1')
		getglobal(OOCAlphaSlider:GetName() .. 'Text'):SetText(L['Fixed/OOC Alpha'])
		getglobal(OOCAlphaSlider:GetName() .. 'Text'):SetTextColor(1,1,0,1)
	
	--Combat Fader Button
		local btnCombatFader = CreateFrame("CheckButton", "YARACombatFaderButton", YARA.panel)
		btnCombatFader:SetWidth(26)
		btnCombatFader:SetHeight(26)
		btnCombatFader:SetPoint("TOPLEFT", OOCAlphaSlider, "BOTTOMLEFT", 0, -20)
		
		btnCombatFader:SetScript("OnShow", function(frame)
			if YARAOpt.CombatFader == 1 then
				frame:SetChecked(true)
			else
				frame:SetChecked(false)
				CombatAlphaSlider:Disable()
				CombatAlphaSlider:SetAlpha(0.3)
			end
		end)
		btnCombatFader:SetScript("OnClick", function(frame)
			local tick = frame:GetChecked()
			if tick then
				PlaySound("igMainMenuOptionCheckBoxOn")
				YARAOpt.CombatFader = 1
				CombatAlphaSlider:Enable()
				CombatAlphaSlider:SetAlpha(1)
			else
				PlaySound("igMainMenuOptionCheckBoxOff")
				YARAOpt.CombatFader = 0
				CombatAlphaSlider:Disable()
				CombatAlphaSlider:SetAlpha(0.3)
			end
		end)

		btnCombatFader:SetHitRectInsets(0, -100, 0, 0)

		btnCombatFader:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Up")
		btnCombatFader:SetPushedTexture("Interface\\Buttons\\UI-CheckBox-Down")
		btnCombatFader:SetHighlightTexture("Interface\\Buttons\\UI-CheckBox-Highlight")
		btnCombatFader:SetCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check")

		local btnCombatFaderText = btnCombatFader:CreateFontString("YARAConfigButton2Title", "ARTWORK", "GameFontNormal")
		btnCombatFaderText:SetPoint("LEFT", btnCombatFader, "RIGHT", 0, 1)
		btnCombatFaderText:SetText((L["Combat Fading"]):format(COMPLAINT_ADDED))
	
	--Combat Alpha Slider
		local CombatAlphaSliderText
		local CombatAlphaSlider = CreateFrame("Slider", "CombatAlphaSlider", YARA.panel, "OptionsSliderTemplate")
		CombatAlphaSlider:SetPoint("TOPLEFT", btnCombatFader, "BOTTOMLEFT", 0, -30)
		CombatAlphaSlider:SetWidth(125)
		CombatAlphaSlider:SetMinMaxValues(0, 1)
		CombatAlphaSlider:SetValueStep(0.1)
		CombatAlphaSlider:SetScript("OnValueChanged", function(self, value)
			YARAOpt.ICalpha = value
			CombatAlphaSliderText:SetText(round(CombatAlphaSlider:GetValue(), 1))
		end)
		CombatAlphaSlider:SetScript("OnShow", function(frame)
			CombatAlphaSlider:SetValue(YARAOpt.ICalpha)
		end)
		
		CombatAlphaSliderText = CombatAlphaSlider:CreateFontString("YARAConfigSlider1Title", "ARTWORK", "GameFontNormal")
		CombatAlphaSliderText:SetPoint("TOP", CombatAlphaSlider, "BOTTOM", 0, 0)
		CombatAlphaSliderText:SetText(round(CombatAlphaSlider:GetValue(), 1))
		
		getglobal(CombatAlphaSlider:GetName() .. 'Low'):SetText('0')
		getglobal(CombatAlphaSlider:GetName() .. 'High'):SetText('1')
		getglobal(CombatAlphaSlider:GetName() .. 'Text'):SetText(L['In-Combat Alpha'])
		getglobal(CombatAlphaSlider:GetName() .. 'Text'):SetTextColor(1,1,0,1)
		
	--CD Peak Alpha Slider
		local CDPeakAlphaSliderText
		local CDPeakAlphaSlider = CreateFrame("Slider", "CDPeakAlphaSlider", YARA.panel, "OptionsSliderTemplate")
		CDPeakAlphaSlider:SetPoint("TOPLEFT", CombatAlphaSlider, "BOTTOMLEFT", 0, -40)
		CDPeakAlphaSlider:SetWidth(125)
		CDPeakAlphaSlider:SetMinMaxValues(0, 1)
		CDPeakAlphaSlider:SetValueStep(0.1)
		CDPeakAlphaSlider:SetScript("OnValueChanged", function(self, value)
			YARAOpt.CDpeak = value
			CDPeakAlphaSliderText:SetText(round(CDPeakAlphaSlider:GetValue(), 1))
		end)
		CDPeakAlphaSlider:SetScript("OnShow", function(frame)
			CDPeakAlphaSlider:SetValue(YARAOpt.CDpeak)
		end)
		
		CDPeakAlphaSliderText = CDPeakAlphaSlider:CreateFontString("YARAConfigSlider1Title", "ARTWORK", "GameFontNormal")
		CDPeakAlphaSliderText:SetPoint("TOP", CDPeakAlphaSlider, "BOTTOM", 0, 0)
		CDPeakAlphaSliderText:SetText(round(CDPeakAlphaSlider:GetValue(), 1))
		
		getglobal(CDPeakAlphaSlider:GetName() .. 'Low'):SetText('0')
		getglobal(CDPeakAlphaSlider:GetName() .. 'High'):SetText('1')
		getglobal(CDPeakAlphaSlider:GetName() .. 'Text'):SetText(L['Rune CD Peak Alpha'])
		getglobal(CDPeakAlphaSlider:GetName() .. 'Text'):SetTextColor(1,1,0,1)
		
	--Rune Rotation Slider
		local RuneRotationSliderText
		local RuneRotationSlider = CreateFrame("Slider", "RuneRotationSlider", YARA.panel, "OptionsSliderTemplate")
		RuneRotationSlider:SetPoint("TOPLEFT", CDPeakAlphaSlider, "BOTTOMLEFT", 0, -40)
		RuneRotationSlider:SetWidth(125)
		RuneRotationSlider:SetMinMaxValues(0, 5)
		RuneRotationSlider:SetValueStep(1)
		RuneRotationSlider:SetScript("OnValueChanged", function(self, value)
			YARAOpt.wedgerad = value
			RuneRotationSliderText:SetText(round(RuneRotationSlider:GetValue(), 0))
			for i = 1, 6 do
				if i == 3 then
					runes[i]:SetRotation((YARAOpt.wedgerad*1.0471976)+1.0471976*4)
				elseif i == 4 then
					runes[i]:SetRotation((YARAOpt.wedgerad*1.0471976)+1.0471976*3)
				else
					runes[i]:SetRotation((YARAOpt.wedgerad*1.0471976)+1.0471976*i)
				end
			end
		end)
		RuneRotationSlider:SetScript("OnShow", function(frame)
			RuneRotationSlider:SetValue(YARAOpt.wedgerad)
		end)
		
		RuneRotationSliderText = RuneRotationSlider:CreateFontString("YARAConfigSlider1Title", "ARTWORK", "GameFontNormal")
		RuneRotationSliderText:SetPoint("TOP", RuneRotationSlider, "BOTTOM", 0, 0)
		RuneRotationSliderText:SetText(round(RuneRotationSlider:GetValue(), 1))
		
		getglobal(RuneRotationSlider:GetName() .. 'Low'):SetText('0')
		getglobal(RuneRotationSlider:GetName() .. 'High'):SetText('5')
		getglobal(RuneRotationSlider:GetName() .. 'Text'):SetText(L['Rune Rotation'])
		getglobal(RuneRotationSlider:GetName() .. 'Text'):SetTextColor(1,1,0,1)
		
		
		
		
-- -----
--	LDB OBJECT/LAUNCHER
-- -----
	LDB:NewDataObject("YARA", {
	type = "launcher",
	icon = [[Interface\AddOns\YARA\img\icon.tga]],
	label = "YARA",
	
	configName = "YARA",
	
	OnClick = function(self, btn)
		if btn == "LeftButton" then
			if YARAFrame:IsMouseEnabled() then
				YARAFrame:EnableMouse(false)
				print("|cffFF0000Y|r|cff00ffffA|r|cff00ff00R|r|cffff00ffA|r: |cff00ffff"..L["Frame is now locked into place!"].."|r")
			else
				YARAFrame:EnableMouse(true)
				print("|cffFF0000Y|r|cff00ffffA|r|cff00ff00R|r|cffff00ffA|r: |cff00ffff"..L["Frame unlocked!"].."|r")
			end
		elseif btn == "RightButton" then
			InterfaceOptionsFrame_OpenToCategory("YARA")
		end
	end,
	
	OnTooltipShow = function(frame)
		frame:AddLine("YARA")
		frame:AddDoubleLine("Left-click:", L["Lock/Unlock YARA"], 0, 1, 0, 1, 1, 1)
		frame:AddDoubleLine("Right-click:", L["Options"], 1, 0, 0, 1, 1, 1)
	end,
})