﻿--[[
	Copyright (c) 2010
	Tobias "Chimaine" Rummelt, kontakt(at)rummelt-software.de

	This software is provided 'as-is', without any express or implied
	warranty. In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
	claim that you wrote the original software. If you use this software
	in a product, an acknowledgment in the product documentation would be
	appreciated but is not required.

	2. Altered source versions must be plainly marked as such, and must not be
	misrepresented as being the original software.

	3. This notice may not be removed or altered from any source
	distribution.
]]

------------------------------------------------------
-- Initialization

local addonName, Addon = ...;
_G[addonName] = Addon;

-- Librarys
LibStub("AceEvent-3.0"):Embed(Addon);

------------------------------------------------------
-- Variables

-- Localisation table
local L = {};

-- Globals
CVENGEANCE_VALUE_UPDATE = "CVENGEANCE_VALUE_UPDATE";
CVENGEANCE_CONFIG_UPDATE = "CVENGEANCE_CONFIG_UPDATE";

-- "Local globals"
local currentAP, maximumAP, percentAP = 0, 0, 0;
local baseHealth = math.huge;

-- Predefine methods
local StartUpdates, StopUpdates, SetUpdateInterval; -- Update Handler
local GetDefaults, GetFormattedAPText, PlayerIsTank, PlayerHasBuff; -- Methods
local OnUpdate, OnMaxHealthChanged, OnTalentsChanged, OnChatCommand; -- Events
local EnableModules, DisableModules; -- Startup

------------------------------------------------------
-- Updates

do
	local UpdateHandler = CreateFrame("Frame");
	local lastUpdate, updateInterval = 0, 1;
	
	function StartUpdates()
		UpdateHandler:SetScript("OnUpdate", function(_, elapsed)
			lastUpdate = lastUpdate + elapsed;
			if (lastUpdate < updateInterval) then return end;
			
			OnUpdate();
			
			lastUpdate = 0;
		end);
	end
	
	function StopUpdates()
		UpdateHandler:SetScript("OnUpdate", nil);
	end
	
	function SetUpdateInterval(interval)
		updateInterval = interval;
	end
end

------------------------------------------------------
-- Locales

do
	local locale = GetLocale();
	
	L["VENGEANCE_NAME"] = select(1, GetSpellInfo(76691))
	L["VENGEANCE_TOOLTIP_LINETEXT"] = ".-(%d+).*"
	
	-- enUS
	L["PLAYER_NO_VENGEANCE"] = "Player is below level 10 or not a tanking class, cVengeance will be disabled on this character."
	L["STATISTIC_TOOLTIP_AVERAGE"] = "Average AP"
	L["STATISTIC_TOOLTIP_HEADER"] = "Statistic for last fight"
	L["STATISTIC_TOOLTIP_HEADER_COMBAT"] = "Statistic for current fight"
	L["STATISTIC_TOOLTIP_MAXIMUM"] = "Highest AP"
	L["STATISTIC_TOOLTIP_UPTIME"] = "Uptime"
	
	if (locale == "frFR") then
		L["PLAYER_NO_VENGEANCE"] = "Le personnage n'a pas atteint le niveau 10 ou n'a pas une classe pouvant tanker, cVengeance sera désactivé pour ce personnage."
		L["STATISTIC_TOOLTIP_AVERAGE"] = "PA moyenne"
		L["STATISTIC_TOOLTIP_HEADER"] = "Statistiques pour le dernier combat"
		L["STATISTIC_TOOLTIP_HEADER_COMBAT"] = "Statistiques pour le combat en cours"
		L["STATISTIC_TOOLTIP_MAXIMUM"] = "PA la plus haute"
		
	elseif (locale == "deDE") then
		L["PLAYER_NO_VENGEANCE"] = "Spieler ist unter Level 10 oder keine Tank Klasse, cVengeance wird auf diesem Charakter deaktiviert."
		L["STATISTIC_TOOLTIP_AVERAGE"] = "Durchschnittliche AS"
		L["STATISTIC_TOOLTIP_HEADER"] = "Statistik für letzten Kampf"
		L["STATISTIC_TOOLTIP_HEADER_COMBAT"] = "Statistik für diesen Kampf"
		L["STATISTIC_TOOLTIP_MAXIMUM"] = "Höchste AS"
		L["STATISTIC_TOOLTIP_UPTIME"] = "Aktiv"
		
	elseif (locale == "koKR") then
	
		
	elseif (locale == "ruRU") then
		
		
	elseif (locale == "esES") then
		L["PLAYER_NO_VENGEANCE"] = "El jugador no ha llegado al nivel 10 o no es una clase de tanqueo, cVengeance se desactivará en este personaje."
		L["STATISTIC_TOOLTIP_AVERAGE"] = "AP medio"
		L["STATISTIC_TOOLTIP_HEADER"] = "Estadísticas de la última lucha"
		L["STATISTIC_TOOLTIP_HEADER_COMBAT"] = "Estadísticas de la lucha actual"
		L["STATISTIC_TOOLTIP_MAXIMUM"] = "Mayor AP"
		
		if (locale == "esMX") then
			
		end
		
	elseif (locale == "zhCN") then
		
		if (locale == "zhTW") then
		
		end	
	end
end

------------------------------------------------------
-- DB defaults

function GetDefaults()
	return {
		profile = {
			barEnabled = true;
			
			width = 150;
			height = 24;
			scale = 1;
					
			locked = false;
			posX = 0;
			posY = 0;
			
			barTooltip = true;
			onlyCombat = false;
					
			barColor = {.77, .12, .23, 1};
			barTexture = "Blizzard";
			
			barBgColor = {0, 0, 0, .5};
			barBgInsets = 0;
					
			barBorderColor = {1, 1, 1, 1};
			barBorder = "Blizzard Tooltip";
			barBorderDistance = 4;
					
			textFormat = "$c / $m ($p%)";
			textFont = "Arial Narrow";
			textSize = 12;
			textColor = {1, 1, 1, 1};
			textStyle = "OUTLINE";
		},
	};
end

------------------------------------------------------
-- Methods

function PlayerIsTank()
	local tree = GetPrimaryTalentTree();
	
	if (not tree) then
		return false;
	else
		return GetTalentTreeRoles(tree) == "TANK";
	end
end

function PlayerHasBuff(name)
	local i = 1;
	while true do
		local match = UnitBuff("player", i);
		
		if (not match) then
			return false; end
		if (match == name) then
			return i; end
			
		i = i + 1;
	end
end

function GetFormattedAPText()
	local text = "" .. Addon.db.profile.textFormat;
	text = gsub(text, "$c", currentAP);
	text = gsub(text, "$m", maximumAP);
	text = gsub(text, "$p", percentAP);
	
	return text;
end

------------------------------------------------------
-- Events

do
	-- Scanning tooltip
	local Scanner = CreateFrame("GameTooltip", addonName .. "Scanner", nil, "GameTooltipTemplate");
	Scanner:SetOwner(WorldFrame, "ANCHOR_NONE");

	function OnUpdate() 	
		local buffIndex = PlayerHasBuff(L["VENGEANCE_NAME"]);
		
		if (not buffIndex) then
			currentAP = 0;
			percentAP = 0;
		else
			Scanner:SetUnitBuff("player", buffIndex);		
			local apText = gsub(cVengeanceScannerTextLeft2:GetText(), L["VENGEANCE_TOOLTIP_LINETEXT"], "%1");
			
			assert(apText ~= nil, "cVengeance: Could not read buff tooltip");
			
			currentAP = tonumber(apText);
			percentAP = floor(((currentAP / maximumAP) * 100) + .5);
		end
		
		Addon:SendMessage(CVENGEANCE_VALUE_UPDATE);
	end
end

function OnMaxHealthChanged(_, unit)
	if (unit ~= "player") then return end;
	
	local stamina = select(2, UnitStat("player", 3));
	local staminaMultiplier = UnitHPPerStamina("player");
	local healthFromStamina = ((stamina - 20) * staminaMultiplier) - 20;

	-- We calculate our base health everytime our max health changes,
	-- because we could have hp% buffs that distort our calculation.
	-- To minimize the chance of disortion, we only keep the lowest value ever seen.
	-- Extra calculation time is acceptable when considering the alternatives.
	baseHealth = min(baseHealth, floor(((UnitHealthMax("player") - healthFromStamina) * .1) + .5));
	
	maximumAP = baseHealth + stamina;
	
	Addon:SendMessage(CVENGEANCE_VALUE_UPDATE);
end

function OnTalentsChanged()
	if (PlayerIsTank()) then
		StartUpdates();
		EnableModules();
	else
		StopUpdates();
		DisableModules();
	end
end

------------------------------------------------------
-- Startup and modules

Addon:RegisterEvent("ADDON_LOADED", function(_, addon)
	if (addon ~= addonName) then return end
	
	Addon:UnregisterEvent("ADDON_LOADED");
	
	-- Level and Class check
	local _, class = UnitClass("player")
	
	if ((UnitLevel("player") < 10) or not ((class == "WARRIOR") or (class == "DEATHKNIGHT") or (class == "DRUID") or (class == "PALADIN"))) then
		print(L["PLAYER_NO_VENGEANCE"]);
		return DisableAddon(addonName);
	end

	Addon.db = LibStub("AceDB-3.0"):New(addonName .. "DB", GetDefaults(), true);
		
	-- Refresh config on reset
	Addon.db.RegisterCallback(Addon, "OnProfileReset", function()
		Addon:SendMessage(CVENGEANCE_CONFIG_UPDATE);
	end);
	
	-- Events and messages
	SetUpdateInterval(.2);
	
	Addon:RegisterEvent("PLAYER_TALENT_UPDATE", OnTalentsChanged);	
	Addon:RegisterEvent("UNIT_MAXHEALTH", OnMaxHealthChanged);
	
	-- Slash command
	local nameUpper = string.upper(addonName);
	
	_G["SLASH_"..nameUpper.."1"] = "/cvengeance";
	_G["SLASH_"..nameUpper.."2"] = "/vengeance";
	_G["SLASH_"..nameUpper.."3"] = "/veng";
	
	SlashCmdList[nameUpper] = function(msg, editBox)
		-- Supply lock command for bar even when options are disabled
		if (msg == "locked") then
			Addon.db.profile.locked = not Addon.db.profile.locked;
			Addon:SendMessage(CVENGEANCE_CONFIG_UPDATE);
		else		
			if (IsAddOnLoaded(addonName .. "_Options")) then
				InterfaceOptionsFrame_OpenToCategory(addonName);
			else
				print(addonName .. ": Options are not enabled. You can still use '/cvengeance locked' to (un-)lock the bar.");
			end
		end
	end;
end)

Addon:RegisterEvent("PLAYER_LOGIN", function()
	OnMaxHealthChanged("UNIT_MAXHEALTH", "player");
end)

function EnableModules()
	Addon["Statistic"].Enable();
	Addon["Statusbar"].Enable();
	Addon["Databroker"].Enable();
end

function DisableModules()
	Addon["Statistic"].Disable();
	Addon["Statusbar"].Disable();
	Addon["Databroker"].Disable();
end

------------------------------------------------------------------------------------------------------------
-- Statistic
------------------------------------------------------------------------------------------------------------

do
	local Statistic = {};

	LibStub("AceEvent-3.0"):Embed(Statistic);
	
	------------------------------------------------------
	-- Variables

	local isRecording = false;

	local maxValue = 0;
	local downtime = 0;

	local startTime = nil;
	local stopTime = nil;
	local lastUpdate = nil;

	local fightData = {};

	local WipeData, CalculateAverage, CalculateUptime;
	local OnValueUpdate, StartRecording, StopRecording;

	------------------------------------------------------
	-- Methods

	function WipeData()
		maxValue = 0;	
		downtime = 0;
		
		startTime = nil;
		stopTime = nil;
		lastUpdate = nil;
		
		wipe(fightData);
	end

	function CalculateAverage()
		if (#fightData == 0) then
			return 0;
		else
			local total = 0;
			
			for _, v in next, fightData do
				total = total + v;
			end
			
			return floor((total / #fightData) + .5);
		end
	end

	function CalculateUptime()
		if (not startTime) then
			return 0;
		else
			-- Are we still recording, calc uptime until now.
			local stopTime = stopTime or GetTime();
			
			local fightLength = stopTime - startTime;
			local uptime = fightLength - downtime;
			
			return floor(((uptime / fightLength) * 100) + .5);
		end
	end

	------------------------------------------------------
	-- Events

	function OnValueUpdate()
		if (not isRecording) then return end;

		-- Downtime
		if (curAP == 0) then
			downtime = downtime + (GetTime() - lastUpdate);
		end
		
		-- New fight maximum?
		if (currentAP > maxValue) then
			maxValue = currentAP;
		end
		
		-- Collect value for average calculation
		fightData[#fightData+1] = currentAP;
		
		-- Record time
		lastUpdate = GetTime();
	end

	function StartRecording()
		if (isRecording) then return end
		
		WipeData();
		
		startTime = GetTime();
		lastUpdate = startTime;
		
		Statistic:RegisterMessage(CVENGEANCE_VALUE_UPDATE, OnValueUpdate);
		
		isRecording = true;
	end

	function StopRecording()
		if (not isRecording) then return end

		stopTime = GetTime();
		Statistic:UnregisterMessage(CVENGEANCE_VALUE_UPDATE);
		
		isRecording = false;
	end

	------------------------------------------------------
	-- Tooltip

	function Statistic.FillTooltip(tooltip)		
		local maxValuePerc = floor(((maxValue / maximumAP) * 100) + .5);
		local average = CalculateAverage();
		local averagePerc = floor(((average / maximumAP) * 100) + .5);
		
		tooltip:AddLine((InCombatLockdown() and L["STATISTIC_TOOLTIP_HEADER_COMBAT"]) or L["STATISTIC_TOOLTIP_HEADER"]);
		tooltip:AddDoubleLine(L["STATISTIC_TOOLTIP_MAXIMUM"], maxValue .. " (" .. maxValuePerc .. "%)", 1, 1, 1, 1, 1, 1);
		tooltip:AddDoubleLine(L["STATISTIC_TOOLTIP_AVERAGE"], average .. " (" .. averagePerc .. "%)", 1, 1, 1, 1, 1, 1);
		tooltip:AddDoubleLine(L["STATISTIC_TOOLTIP_UPTIME"], CalculateUptime() .. "%", 1, 1, 1, 1, 1, 1);
	end

	------------------------------------------------------
	-- Startup and Disable

	function Statistic.Enable()
		if ( InCombatLockdown() ) then
			StartRecording();
		end

		Statistic:RegisterEvent("PLAYER_REGEN_DISABLED", StartRecording);
		Statistic:RegisterEvent("PLAYER_REGEN_ENABLED", StopRecording);
	end

	function Statistic.Disable()
		StopRecording();
		WipeData();

		Statistic:UnregisterEvent("PLAYER_REGEN_DISABLED");
		Statistic:UnregisterEvent("PLAYER_REGEN_ENABLED");
	end
	
	-- Register Module
	Addon["Statistic"] = Statistic;
end

------------------------------------------------------------------------------------------------------------
-- Statusbar
------------------------------------------------------------------------------------------------------------

do
	local Statusbar = {};

	local LSM = LibStub("LibSharedMedia-3.0");
	LibStub("AceEvent-3.0"):Embed(Statusbar);
	
	------------------------------------------------------
	-- Variables

	local bar = CreateFrame("StatusBar", "cVengeanceStatusBar", UIParent);

	local BarOnMouseEnter, BarOnMouseLeave, BarOnMouseDown, BarOnMouseUp;
	local OnEnteringCombat, OnLeavingCombat, OnValueUpdate;
	local SetRectangle, OnConfigUpdate;

	------------------------------------------------------
	-- Override bar.Show
	
	local orgShow = bar.Show;
	
	function bar.Show()
		if (Addon.db.profile.barEnabled == true) then
			orgShow(bar);
		else
			bar:Hide();
		end
	end

	------------------------------------------------------
	-- Frame moving

	function BarOnMouseEnter()	
		GameTooltip:SetOwner(bar, "ANCHOR_BOTTOMRIGHT");
		GameTooltip:ClearLines();
		
		Addon["Statistic"].FillTooltip(GameTooltip);
		
		GameTooltip:Show();
	end

	function BarOnMouseLeave()
		GameTooltip:Hide();
	end

	function BarOnMouseDown(_, button)
		if (button ~= "LeftButton") then return end;
		
		bar:StartMoving();
	end

	function BarOnMouseUp()
		bar:StopMovingOrSizing();
		
		-- We need to do this, because f:StartMoving() sets this to true, and the position
		-- will be saved on Blizzard's side. This could cause issues with positioning.
		bar:SetUserPlaced(false);
		
		local parentX, parentY = UIParent:GetCenter();
		local x, y = bar:GetCenter();
		
		Addon.db.profile.posX = x - parentX;
		Addon.db.profile.posY = y - parentY;
		
		SetRectangle();
		
		-- Update GUI if it's shown
		LibStub("AceConfigRegistry-3.0", true):NotifyChange(addonName);
	end

	------------------------------------------------------
	-- Config

	function SetRectangle()
		bar:SetPoint("CENTER", UIParent, "CENTER", Addon.db.profile.posX, Addon.db.profile.posY);
		bar:SetWidth(Addon.db.profile.width);
		bar:SetHeight(Addon.db.profile.height);
		bar:SetScale(Addon.db.profile.scale);
	end

	function OnConfigUpdate()
		-- Rectangle
		SetRectangle();
		
		-- Backdrop and Border
		bar:SetBackdrop({
			bgFile = "Interface\\Buttons\\WHITE8X8",
			insets = {
				left = Addon.db.profile.barBgInsets,
				right = Addon.db.profile.barBgInsets,
				top = Addon.db.profile.barBgInsets,
				bottom = Addon.db.profile.barBgInsets,
			}
		});
		
		bar:SetBackdropColor(unpack(Addon.db.profile.barBgColor));		
		bar:SetStatusBarTexture(LSM:Fetch("statusbar", Addon.db.profile.barTexture));
		bar:SetStatusBarColor(unpack(Addon.db.profile.barColor));
		
		bar.overlay:ClearAllPoints();
		bar.overlay:SetPoint("TOPLEFT", -Addon.db.profile.barBorderDistance, Addon.db.profile.barBorderDistance);
		bar.overlay:SetPoint("BOTTOMRIGHT", Addon.db.profile.barBorderDistance, -Addon.db.profile.barBorderDistance);
		
		bar.overlay:SetBackdrop({
			edgeFile = LSM:Fetch("border", Addon.db.profile.barBorder),
			edgeSize = 16,
		});
		
		bar.overlay:SetBackdropBorderColor(unpack(Addon.db.profile.barBorderColor));
		
		-- Font
		bar.text:SetFont(LSM:Fetch("font", Addon.db.profile.textFont), Addon.db.profile.textSize, Addon.db.profile.textStyle);
		bar.text:SetTextColor(unpack(Addon.db.profile.textColor));
		
		-- Bar lock
		local unlock = not Addon.db.profile.locked;

		bar:SetMovable(unlock);
		
		bar:SetScript("OnMouseDown", (unlock and BarOnMouseDown) or nil);
		bar:SetScript("OnMouseUp", (unlock and BarOnMouseUp) or nil);
		
		-- Do we have a tooltip?
		local tooltip = Addon.db.profile.barTooltip;
		
		bar:SetScript("OnEnter", (tooltip and BarOnMouseEnter) or nil);
		bar:SetScript("OnLeave", (tooltip and BarOnMouseLeave) or nil);
		
		-- OOC auto hide
		if ( Addon.db.profile.onlyCombat ) then
			Statusbar:RegisterEvent("PLAYER_REGEN_DISABLED", OnEnteringCombat);
			Statusbar:RegisterEvent("PLAYER_REGEN_ENABLED", OnLeavingCombat);
			
			bar:Hide();
		else		
			Statusbar:UnregisterEvent("PLAYER_REGEN_DISABLED");
			Statusbar:UnregisterEvent("PLAYER_REGEN_ENABLED");
			
			bar:Show();
		end	
	end

	------------------------------------------------------
	-- Events

	function OnEnteringCombat()
		bar:Show();
	end

	function OnLeavingCombat()
		bar:Hide();
	end

	function OnValueUpdate()		
		bar.text:SetText(GetFormattedAPText());
		
		bar:SetMinMaxValues(0, maximumAP);
		bar:SetValue(currentAP);
	end

	------------------------------------------------------
	-- Startup and Disable

	function Statusbar.Enable()		
		Statusbar:RegisterMessage(CVENGEANCE_VALUE_UPDATE, OnValueUpdate);
		Statusbar:RegisterMessage(CVENGEANCE_CONFIG_UPDATE, OnConfigUpdate);
		
		OnConfigUpdate();
	end

	function Statusbar.Disable()		
		Statusbar:UnregisterAllEvents();
		Statusbar:UnregisterAllMessages();
		
		bar:Hide();
	end

	------------------------------------------------------
	-- Initialize
	do
		bar:Hide();
		
		bar:SetClampedToScreen(true);
		bar:EnableMouse(true);
		
		bar.text = bar:CreateFontString("$parentText");
		bar.text:SetPoint("CENTER");
		
		bar.overlay = CreateFrame("Frame", nil, bar);
	end
	
	-- Register Module
	Addon["Statusbar"] = Statusbar;
end

------------------------------------------------------------------------------------------------------------
-- Databroker feed
------------------------------------------------------------------------------------------------------------

do
	local Databroker = {};

	LibStub("AceEvent-3.0"):Embed(Databroker);
	
	------------------------------------------------------
	-- Variables

	local dataObj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("cVengeance", {
		type = "data source",
		text = "0 AP",
		value = "0",
		suffix = "AP",
		OnClick = function()
			InterfaceOptionsFrame_OpenToCategory(addonName)
		end,
	});
	
	local OnValueUpdate, OnTooltipShow;

	------------------------------------------------------
	-- Events
	
	function OnValueUpdate()				
		dataObj.value = GetFormattedAPText();
		dataObj.text = dataObj.value .. " " .. dataObj.suffix;
	end

	function dataObj.OnTooltipShow(tooltip)
		Addon["Statistic"].FillTooltip(tooltip);
	end
	
	------------------------------------------------------
	-- Startup and Disable

	function Databroker.Enable()		
		Databroker:RegisterMessage(CVENGEANCE_VALUE_UPDATE, OnValueUpdate);
	end
	
	function Databroker.Disable()
		Databroker:UnregisterMessage(CVENGEANCE_VALUE_UPDATE);
	end
	
	-- Register Module
	Addon["Databroker"] = Databroker;
end
