﻿--[[
	Copyright (c) 2010
	Tobias "Chimaine" Rummelt, kontakt(at)rummelt-software.de

	This software is provided 'as-is', without any express or implied
	warranty. In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
	claim that you wrote the original software. If you use this software
	in a product, an acknowledgment in the product documentation would be
	appreciated but is not required.

	2. Altered source versions must be plainly marked as such, and must not be
	misrepresented as being the original software.

	3. This notice may not be removed or altered from any source
	distribution.
]]

------------------------------------------------------
-- Initialization

local addonName, Addon = "cVengeance", _G["cVengeance"];

------------------------------------------------------
-- Variables

-- Localisation table
local L = {};

-- Predefine methods
local GetOption, GetColorOption, GetOptionAsString;
local SetOption, SetColorOption, SetOptionAsNumber;
local GetOptionTable, GetProfileTable;

------------------------------------------------------
-- Librarys

local LSM = LibStub("LibSharedMedia-3.0");

------------------------------------------------------
-- Locales

do
	local locale = GetLocale();
	
	L["VENGEANCE_NAME"] = select(1, GetSpellInfo(76691))
	L["VENGEANCE_TOOLTIP_LINETEXT"] = ".-(%d+).*"
	
	-- enUS
	L["BAR_BACKGROUND_COLOR"] = "Bar background"
	L["BAR_BACKGROUND_COLOR_TOOLTIP"] = "The color of the statusbar background"
	L["BAR_BACKGROUND_INSETS"] = "Background insets"
	L["BAR_BACKGROUND_INSETS_TOOLTIP"] = "The distance of the background from the border"
	L["BAR_BORDER_COLOR"] = "Border color"
	L["BAR_BORDER_COLOR_TOOLTIP"] = "The color of the statusbar border"
	L["BAR_BORDER_DISTANCE"] = "Border distance"
	L["BAR_BORDER_DISTANCE_TOOLTIP"] = "The distance of the border to the bar"
	L["BAR_BORDER_TEXTURE"] = "Border texture"
	L["BAR_BORDER_TEXTURE_TOOLTIP"] = "The texture of the statusbar border"
	L["BAR_COLOR"] = "Statusbar color"
	L["BAR_COLOR_TOOLTIP"] = "The color of the statusbar"
	L["BAR_DIMENSION_HEIGHT"] = "Height"
	L["BAR_DIMENSION_HEIGHT_TOOLTIP"] = "The height of the statusbar"
	L["BAR_DIMENSION_SCALE"] = "Scale"
	L["BAR_DIMENSION_SCALE_TOOLTIP"] = "The scale of the statusbar"
	L["BAR_DIMENSION_WIDTH"] = "Width"
	L["BAR_DIMENSION_WIDTH_TOOLTIP"] = "The width of the statusbar"
	L["BAR_ENABLE"] = "Enable statusbar"
	L["BAR_ENABLE_TOOLTIP"] = "Enables the statusbar"
	L["BAR_LOCK"] = "Lock statusbar"
	L["BAR_LOCK_TOOLTIP"] = "Lock or unlock the statusbar"
	L["BAR_ONLY_COMBAT"] = "Only combat"
	L["BAR_ONLY_COMBAT_TOOLTIP"] = "Hide the statusbar when out of combat"
	L["BAR_POSITION_X"] = "X position"
	L["BAR_POSITION_X_TOOLTIP"] = "The horizontal position of the statusbar"
	L["BAR_POSITION_Y"] = "Y position"
	L["BAR_POSITION_Y_TOOLTIP"] = "The vertical position of the statusbar"
	L["BAR_TEXTURE"] = "Statusbar texture"
	L["BAR_TEXTURE_TOOLTIP"] = "The texture of the statusbar"
	L["BAR_TOOLTIP"] = "Statusbar tooltip"
	L["BAR_TOOLTIP_TOOLTIP"] = "Shows a statistic tooltip on statusbar mouseover"
	L["HEADER"] = "Options"
	L["HEADER_APPEARANCE"] = "Statusbar appearance"
	L["HEADER_DIMENSIONS"] = "Statusbar dimensions"
	L["HEADER_POSITIONING"] = "Statusbar positioning"
	L["HEADER_TEXT"] = "Text"
	L["TEXT_COLOR"] = "Text color"
	L["TEXT_COLOR_TOOLTIP"] = "The color of the text"
	L["TEXT_FONT"] = "Font"
	L["TEXT_FONT_TOOLTIP"] = "Font face of the statusbar text"
	L["TEXT_FORMAT"] = "Text format"
	L["TEXT_FORMAT_TOOLTIP"] = "The text on the statusbar and broker plugin. Leave blank to hide text. Substitutions are: %c : current value | %m : maximum value | %p : percent value"
	L["TEXT_SIZE"] = "Text size"
	L["TEXT_SIZE_TOOLTIP"] = "The size of the statusbar text"
	L["TEXT_STYLE"] = "Text outline"
	L["TEXT_STYLE_TOOLTIP"] = "The thickness of the text outline"
	
	if (locale == "frFR") then
		
	elseif (locale == "deDE") then
		
	elseif (locale == "koKR") then
		
	elseif (locale == "ruRU") then
		
	elseif ((locale == "esES") or (locale == "esMX")) then
		
	elseif (locale == "zhCN") then
	
	elseif (locale == "zhTW") then
	
	end
end

------------------------------------------------------
-- Methods

function GetOption(info)
	return Addon.db.profile[info[#info]];
end

function SetOption(info, value)
	Addon.db.profile[info[#info]] = value;
	
	Addon:SendMessage(CVENGEANCE_CONFIG_UPDATE);
end

function GetOptionAsString(info)
	return tostring(GetOption(info));
end

function SetOptionAsNumber(info, value)
	SetOption(info, tonumber(value));
end

function GetColorOption(info)
	return unpack(GetOption(info));
end

function SetColorOption(info, r, g, b, a)
	SetOption(info, { r, g, b, a });
end

------------------------------------------------------
-- Option table

local optionTable, profileTable = nil, nil;

function GetOptionTable()
	if ( not optionTable ) then	
		optionTable = {
			type = "group",
			name = "|cff7696bcc|rVengeance " .. L["HEADER"],
			get = GetOption,
			set = SetOption,
			args = {
				barEnabled = {
					order = 1,
					type = "toggle",
					name = L["BAR_ENABLE"],
					desc = L["BAR_ENABLE_TOOLTIP"],
					width = "full",
				},
				locked = {
					order = 2,
					type = "toggle",
					name = L["BAR_LOCK"],
					desc = L["BAR_LOCK_TOOLTIP"],
					width = "full",
					disabled = function()
						return not Addon.db.profile.barEnabled;
					end,
				},
				onlyCombat = {
					order = 3,
					type = "toggle",
					name = L["BAR_ONLY_COMBAT"],
					desc = L["BAR_ONLY_COMBAT_TOOLTIP"],
					width = "full",
					disabled = function()
						return not Addon.db.profile.barEnabled;
					end,
				},
				barTooltip = {
					order = 4,
					type = "toggle",
					name = L["BAR_TOOLTIP"],
					desc = L["BAR_TOOLTIP_TOOLTIP"],
					width = "full",
					disabled = function()
						return not Addon.db.profile.barEnabled;
					end,
				},
				
				position = {
					order = 10,
					type = "group",
					name = L["HEADER_POSITIONING"],
					inline = true,
					args = {
						posX = {
							order = 1,
							type = "input",
							name = L["BAR_POSITION_X"],
							desc = L["BAR_POSITION_X_TOOLTIP"],
							dialogControl = "NumberEditBox",
							get = GetOptionAsString,
							set = SetOptionAsNumber,
						},
						posY = {
							order = 2,
							type = "input",
							name = L["BAR_POSITION_Y"],
							desc = L["BAR_POSITION_Y_TOOLTIP"],
							dialogControl = "NumberEditBox",
							get = GetOptionAsString,
							set = SetOptionAsNumber,
						},
					},
				}, -- position
				
				dimensions = {
					order = 20,
					type = "group",
					name = L["HEADER_DIMENSIONS"],
					inline = true,
					args = {
						scale = {
							order = 1,
							type = "range",
							name = L["BAR_DIMENSION_SCALE"],
							desc = L["BAR_DIMENSION_SCALE_TOOLTIP"],
							min = 0.1,
							max = 5,
							bigStep = 0.1,
						},
						width = {
							order = 2,
							type = "range",
							name = L["BAR_DIMENSION_WIDTH"],
							desc = L["BAR_DIMENSION_WIDTH_TOOLTIP"],
							min = 10,
							softMin = 50,
							softMax = 500,
							bigStep = 1,
						},
						height = {
							order = 3,
							type = "range",
							name = L["BAR_DIMENSION_HEIGHT"],
							desc = L["BAR_DIMENSION_HEIGHT_TOOLTIP"],
							min = 10,
							softMin = 10,
							softMax = 100,
							bigStep = 1,
						},
					},
				}, -- dimensions				
				
				appearance = {
					order = 30,
					type = "group",
					name = L["HEADER_APPEARANCE"],
					inline = true,
					args = {
						barBgColor = {
							order = 1,
							type = "color",
							name = L["BAR_BACKGROUND_COLOR"],
							desc = L["BAR_BACKGROUND_COLOR_TOOLTIP"],
							hasAlpha = true,
							get = GetColorOption,
							set = SetColorOption,
						},		
						barBgInsets = {
							order = 2,
							type = "range",
							name = L["BAR_BACKGROUND_INSETS"],
							desc = L["BAR_BACKGROUND_INSETS_TOOLTIP"],
							softMin = -10,
							softMax = 10,
							bigStep = 1,
						},
						barColor = {
							order = 3,
							type = "color",
							name = L["BAR_COLOR"],
							desc = L["BAR_COLOR_TOOLTIP"],
							hasAlpha = true,
							get = GetColorOption,
							set = SetColorOption,
						},
						barTexture = {
							order = 4,
							type = "select",
							name = L["BAR_TEXTURE"],
							desc = L["BAR_TEXTURE_TOOLTIP"],
							dialogControl = "LSM30_Statusbar",
							values = LSM:HashTable("statusbar"),
						},
						barBorderColor = {
							order = 5,
							type = "color",
							name = L["BAR_BORDER_COLOR"],
							desc = L["BAR_BORDER_COLOR_TOOLTIP"],
							hasAlpha = true,
							get = GetColorOption,
							set = SetColorOption,
						},
						barBorder = {
							order = 6,
							type = "select",
							name = L["BAR_BORDER_TEXTURE"],
							desc = L["BAR_BORDER_TEXTURE_TOOLTIP"],
							dialogControl = "LSM30_Border",
							values = LSM:HashTable("border"),
						},
						barBorderDistance = {
							order = 7,
							type = "range",
							name = L["BAR_BORDER_DISTANCE"],
							desc = L["BAR_BORDER_DISTANCE_TOOLTIP"],
							softMin = -10,
							softMax = 10,
							bigStep = 1,
						},
					},
				}, -- appearance				
				
				font = {
					order = 40,
					type = "group",
					name = L["HEADER_TEXT"],
					inline = true,
					args = {
						textFormat = {
							order = 1,
							type = "input",
							name = L["TEXT_FORMAT"],
							desc = L["TEXT_FORMAT_TOOLTIP"],
						},
						textFont = {
							order = 2,
							type = "select",
							name = L["TEXT_FONT"],
							desc = L["TEXT_FONT_TOOLTIP"],
							dialogControl = "LSM30_Font",
							values = LSM:HashTable("font"),
						},	
						textColor = {
							order = 4,
							type = "color",
							name = L["TEXT_COLOR"],
							desc = L["TEXT_COLOR_TOOLTIP"],
							hasAlpha = true,
							get = GetColorOption,
							set = SetColorOption,
							--width = "half",
						},				
						textStyle = {
							order = 5,
							type = "select",
							name = L["TEXT_STYLE"],
							desc = L["TEXT_STYLE_TOOLTIP"],
							values = { ["NONE"] = "None", ["OUTLINE"] = "Thin", ["THICKOUTLINE"] = "Thick", },
							--width = "half",
						},
						textSize = {
							order = 6,
							type = "range",
							name = L["TEXT_SIZE"],
							desc = L["TEXT_SIZE_TOOLTIP"],
							min = 1,
							max = 100,
							softMin = 2,
							softMax = 40,
							bigStep = 1,
						},
					},
				}, -- font
			},
		};
	end
	
	return optionTable;	
end

function GetProfileTable()
	if (not profileTable) then
		profileTable = LibStub("AceDBOptions-3.0"):GetOptionsTable(Addon.db);
	end
		
	return profileTable;
end

do
	LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, GetOptionTable);
	LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName .. "Profiles", GetProfileTable);
	
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName);
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName .. "Profiles", "Profiles", addonName);
end
